#define DEBUG

#region usings
using System;
using System.ComponentModel.Composition;
using System.IO.Ports;
using System.Collections.Generic;



using VVVV.PluginInterfaces.V1;
using VVVV.PluginInterfaces.V2;
using VVVV.Utils.VColor;
using VVVV.Utils.VMath;

using VVVV.Core.Logging;
using uPLibrary.Hardware;
using uPLibrary.Nfc;


#endregion usings


namespace VVVV.Nodes
{
	#region PluginInfo
	[PluginInfo(Name = "uNFC", Category = "Nfc, RFID Reader", Help = "uses uNFC Library for PN532", Tags = "",AutoEvaluate = true)]
	#endregion PluginInfo
	public class ValueuNFCNode : IPluginEvaluate, IDisposable
	{
		#region fields & pins
		[Input("Reset", DefaultValue = 0, IsBang = true)]
		public ISpread<bool> FInit;
		
		[Input("Port", EnumName = "ComPortEnum")]
        public IDiffSpread<EnumEntry> FPort;
		
		[Input("Enabled", DefaultValue = 0.0, IsSingle = true)]
		ISpread<bool> FEnabled;

		[Output("DetectedID")]
		public ISpread<String> FOutput;

		[Output("Type")]
		public ISpread<String> FType;
		
		[Output("Connected", DefaultValue = 0)]
		public ISpread<bool> FConn;

		[Import()]
		public ILogger FLogger;
		#endregion fields & pins
		
		private uPLibrary.Nfc.INfcReader nfc;
		private uPLibrary.Hardware.Nfc.PN532CommunicationHSU hsuLayer;
		
		

		// we make a new list and called it myList 
		List<uPLibrary.Hardware.Nfc.PN532CommunicationHSU> myList = new List<uPLibrary.Hardware.Nfc.PN532CommunicationHSU>();


		
		//add comport entries to the enum in the constructor
		[ImportingConstructor]
		public ValueuNFCNode()
		{ 
			EnumManager.UpdateEnum("ComPortEnum", 
					System.IO.Ports.SerialPort.GetPortNames()[0], System.IO.Ports.SerialPort.GetPortNames());
		}
		
		//called when data for any output pin is requested
		public void Evaluate(int SpreadMax)
		{
			
			FOutput.SliceCount = 1;
			FType.SliceCount = 1;
			FConn.SliceCount = 1;
			
			
			
				
				if(FEnabled[0]){

					
					if(FInit[0]||FPort.IsChanged){
					
						//release everything if in use
						Dispose();
							
						
						
						//connect everything according to unfc library
						hsuLayer = new uPLibrary.Hardware.Nfc.PN532CommunicationHSU(FPort[0]);
						
						#if DEBUG
						FLogger.Log(LogType.Debug, "Port Open");
						#endif
						
						nfc = new uPLibrary.Nfc.NfcPN532Reader(hsuLayer);
						
						//subscribe to events
						nfc.TagDetected += nfc_TagDetected;	
						nfc.TagLost += nfc_TagLost;
						nfc.Open(NfcTagType.MifareUltralight);
						
						FConn[0] = true;
						
		
					}
				}
				else Dispose();
				
			
			
		

			//FLogger.Log(LogType.Debug, "hm");
		}
		//dispose everything on delete
		
		public void Dispose()
		{

				if(hsuLayer != null){
					
					nfc.Close();
					FConn[0] = false;
					hsuLayer.port.Close();
					hsuLayer = null;
					
					#if DEBUG
					FLogger.Log(LogType.Debug, "Port Closed");
					#endif
					
				}
				
				
		}
		
 
	    static private string hexChars = "0123456789ABCDEF"; 
	 
	    /// <summary> 
	    /// Convert hex byte array in a hex string 
	    /// </summary> 
	    /// <param name="value">Byte array with hex values</param> 
	    /// <returns>Hex string</returns> 
		
		
		//define what happens if tag is lost
		private void nfc_TagLost(object sender, NfcTagEventArgs e) 
	    { 
	        FType[0] = null;
	    	FOutput[0] = null;
	    	#if DEBUG
				FLogger.Log(LogType.Debug, "Tag lost");
			#endif
	    	
	    	
	    } 
		
		
		//define what happens if tag is detected
	     private void nfc_TagDetected(object sender, NfcTagEventArgs e) 
	    { 
	 		
	    	FType[0] =e.NfcTagType.ToString();
	    	FOutput[0] = HexToString(e.Connection.ID);
	    	#if DEBUG
				FLogger.Log(LogType.Debug, "Tag detected");
			#endif
	  	
	        
	    } 
	    static internal string HexToString(byte[] value) 
	    { 
	        System.Text.StringBuilder hexString = new System.Text.StringBuilder(); 
	        for (int i = 0; i < value.Length; i++) 
	        { 
	        hexString.Append(hexChars[(value[i] >> 4) & 0x0F]); 
	        hexString.Append(hexChars[value[i] & 0x0F]); 
	        } 
	        return hexString.ToString(); 
   		} 
	}
	

	
}
